package ij.plugin;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;

import ij.gui.GenericDialog;
import ij.gui.ImageCanvas;
import ij.gui.Toolbar;

import ij.process.ByteProcessor;
import ij.process.ColorProcessor;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;

import ij.measure.Calibration;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import java.awt.Graphics;
import java.awt.Event;

/** Implements the Image/Stacks/Make Montage command. */
public class MontageMaker extends JComponent implements MouseListener, MouseMotionListener, PlugIn {

        private static int columns, rows, first, last, inc, borderWidth;
        private static double scale, mag;
        private static boolean label;
        private static boolean useForegroundColor;
        private static int saveID;
        private static Rectangle[] imageBound = {};
        private ImageCanvas  win;
        private StackEditor editor=new StackEditor();
        private ImageProcessor montage;
        private ImagePlus imp2;
        private ImagePlus imp;
        private ImageProcessor slice=null;
        private ImagePlus ip = null;
        private int num = 0;
        private boolean pressed = false;
        private Toolbar toolbar;

        public void run(String arg) {
                imp = WindowManager.getCurrentImage();
                if (imp==null || imp.getStackSize()==1)
                        {IJ.error("Stack required"); return;}
                makeMontage(imp);
                saveID = imp.getID();
        IJ.register(MontageMaker.class);
        }

        public void makeMontage(ImagePlus imp) {
                        int nSlices = imp.getStackSize();
                        imageBound=new Rectangle[nSlices+1];
                        if (columns==0 || imp.getID()!=saveID) {
                                columns = (int)Math.sqrt(nSlices);
                                rows = columns;
                                int n = nSlices - columns*rows;
                                if (n>0) columns += (int)Math.ceil((double)n/rows);
                                scale = 1.0;
                                if (imp.getWidth()*columns>800)
                                        scale = 0.5;
                                if (imp.getWidth()*columns>1600)
                                        scale = 0.25;
                                inc = 1;
                                first = 1;
                                last = nSlices;
                        }

                        GenericDialog gd = new GenericDialog("Make Montage", IJ.getInstance());
                        gd.addNumericField("Columns:", columns, 0);
                        gd.addNumericField("Rows:", rows, 0);
                        gd.addNumericField("Scale Factor:", scale, 2);
                        gd.addNumericField("First Slice:", first, 0);
                        gd.addNumericField("Last Slice:", last, 0);
                        gd.addNumericField("Increment:", inc, 0);
                        gd.addNumericField("Border Width:", borderWidth, 0);
                        gd.addCheckbox("Label Slices", label);
                        gd.addCheckbox("Use Foreground Color", useForegroundColor);
                        gd.showDialog();
                        if (gd.wasCanceled())
                                return;
                        columns = (int)gd.getNextNumber();
                        rows = (int)gd.getNextNumber();
                        scale = gd.getNextNumber();
                        first = (int)gd.getNextNumber();
                        last = (int)gd.getNextNumber();
                        inc = (int)gd.getNextNumber();
                        borderWidth = (int)gd.getNextNumber();
                        if (borderWidth<0) borderWidth = 0;
                        if (first<1) first = 1;
                        if (last>nSlices) last = nSlices;
                        if (first>last)
                                {first=1; last=nSlices;}
                        if (inc<1) inc = 1;
                        if (gd.invalidNumber()) {
                                IJ.error("Invalid number");
                                return;
                        }
                        label = gd.getNextBoolean();
                        useForegroundColor = gd.getNextBoolean();
                        makeMontage(imp, columns, rows, scale, first, last, inc, borderWidth, label);
        }

        public void makeMontage(ImagePlus imp, int columns, int rows, double scale, int first, int last, int inc, int borderWidth, boolean labels) {
                int stackWidth = imp.getWidth();
                int stackHeight = imp.getHeight();
                int nSlices = imp.getStackSize();
                int width = (int)(stackWidth*scale);
                int height = (int)(stackHeight*scale);
                int montageWidth = width*columns;
                int montageHeight = height*rows;
                ImageProcessor ip = imp.getProcessor();
                montage = ip.createProcessor(montageWidth+borderWidth/2, montageHeight+borderWidth/2);
                Color fgColor=Color.white;
                Color bgColor = Color.black;
                if (useForegroundColor) {
                        fgColor = Toolbar.getForegroundColor();
                        bgColor = Toolbar.getBackgroundColor();
                } else {
                        boolean whiteBackground = false;
                        if ((ip instanceof ByteProcessor) || (ip instanceof ColorProcessor)) {
                                ImageStatistics is = imp.getStatistics();
                                whiteBackground = is.mode>=200;
                                if (imp.isInvertedLut())
                                        whiteBackground = !whiteBackground;
                        }
                        if (whiteBackground) {
                                fgColor=Color.black;
                                bgColor = Color.white;
                        }
                }
                montage.setColor(bgColor);
                montage.fill();
                montage.setColor(fgColor);
                ImageStack stack = imp.getStack();
                int x = 0;
                int y = 0;
                int index = 0;
                ImageProcessor aSlice;
            int slice = first;
                while (slice<=last) {
                        aSlice = stack.getProcessor(slice);
                        if (scale!=1.0){
                            aSlice = aSlice.resize(width, height);
                        }
                        imageBound[index] = new Rectangle(x, y, width, height);
                        //System.out.println("imageBound["+index +"]=" + imageBound[index]);
                        montage.insert(aSlice, x, y);
                        String label = stack.getShortSliceLabel(slice);
                        if (borderWidth>0) drawBorder(montage, x, y, width, height, borderWidth);
                        if (labels) drawLabel(montage, slice, label, x, y, width, height);
                        x += width;
                        if (x>=montageWidth) {
                                x = 0;
                                y += height;
                                if (y>=montageHeight)
                                        break;
                        }
                        IJ.showProgress((double)(slice-first)/(last-first));
                        slice += inc;index++;
                }
                if (borderWidth>0) {
                        int w2 = borderWidth/2;
                        drawBorder(montage, w2, w2, montageWidth-w2, montageHeight-w2, borderWidth);
                }
                IJ.showProgress(1.0);
                imp2 = new ImagePlus("Montage", montage);
                imp2.setCalibration(imp.getCalibration());
                Calibration cal = imp2.getCalibration();
                if (cal.scaled()) {
                        cal.pixelWidth /= scale;
                        cal.pixelHeight /= scale;
                }
        imp2.setProperty("Info", "xMontage="+columns+"\nyMontage="+rows+"\n");
                imp2.show();
                win = imp2.getCanvas();
                win.addMouseListener(this);
                win.addMouseMotionListener(this);
                mag = win.getMagnification();

                //win.repaint();
                if(mag != 1.0)//IJ.showMessage(" 100 % zoom requied ! \n automaticly zoom to 100% \n press OK");
                IJ.doCommand("View 100%");
        }

        void drawBorder(ImageProcessor montage, int x, int y, int width, int height, int borderWidth) {
                montage.setLineWidth(borderWidth);
                montage.moveTo(x, y);
                montage.lineTo(x+width, y);
                montage.lineTo(x+width, y+height);
                montage.lineTo(x, y+height);
                montage.lineTo(x, y);
        }

        void drawLabel(ImageProcessor montage, int slice, String label, int x, int y, int width, int height) {
                if (label!=null && !label.equals("") && montage.getStringWidth(label)>=width) {
                        do {
                                label = label.substring(0, label.length()-1);
                        } while (label.length()>1 && montage.getStringWidth(label)>=width);
                }
                if (label==null || label.equals(""))
                        label = ""+slice;
                int swidth = montage.getStringWidth(label);
                x += width/2 - swidth/2;
                y += height;
                montage.moveTo(x, y);
                montage.drawString(label);
        }
        public void update(Graphics g) {;
                paintComponent(g);
        }


    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        /*ImageStack stack = imp.getStack();
        double mag = win.getMagnification();
        int stackWidth = imp.getWidth();
        int stackHeight = imp.getHeight();
        int width = (int)(stackWidth*mag);
        int height = (int)(stackHeight*mag);
        int montageWidth = width*columns;
        int montageHeight = height*rows;
        int x = 0;
        int y = 0;
        int index = 0;
        ImageProcessor aSlice;
    int slice = first;
        while (slice<=last) {
                aSlice = stack.getProcessor(slice);
                if (scale!=1.0){
                    aSlice = aSlice.resize(width, height);
                }
                imageBound[index] = new Rectangle(x, y, width, height);
                //System.out.println("imageBound["+index +"]=" + imageBound[index]);
                montage.insert(aSlice, x, y);
                String label = stack.getShortSliceLabel(slice);
                if (borderWidth>0) drawBorder(montage, x, y, width, height, borderWidth);
                //if (labels) drawLabel(montage, slice, label, x, y, width, height);
                x += width;
                if (x>=montageWidth) {
                        x = 0;
                        y += height;
                        if (y>=montageHeight)
                                break;
                }
                IJ.showProgress((double)(slice-first)/(last-first));
                slice += inc;index++;
        }*/
    }
        public void mousePressed(MouseEvent e) {
            Point p = e.getPoint();
            ImagePlus thisimp = WindowManager.getCurrentImage();
            ImageCanvas ic = thisimp.getCanvas();
            double thismag = ic.getMagnification();
            int flags = e.getModifiers();
            int toolID = Toolbar.getToolId();
            ImageStack stack = imp.getStack();
            int nSlices = imp.getStackSize();

                if (toolID!=Toolbar.MAGNIFIER) {
                for (int i = 0; i < nSlices; i++) {
                    slice = stack.getProcessor(i + 1);
                    if (imageBound[i] != null &&
                        imageBound[i].contains(e.getPoint())) {
                        imp2.setRoi(imageBound[i]);
                        num = i + 1;
                        break;
                    }
                }
            }
            else if (toolID==Toolbar.MAGNIFIER) {
                if(((flags & Event.META_MASK)!=0))
                    mag=1.0;//System.out.println("((flags & Event.META_MASK)!=0)="+((flags & Event.META_MASK)!=0));
                if(thismag > mag)IJ.runPlugIn("ij.plugin.Zoom", "out");
                else if(thismag < mag)IJ.runPlugIn("ij.plugin.Zoom", "in");
                else IJ.runPlugIn("ij.plugin.Zoom", "orig");
                toolbar=Toolbar.getInstance();
                toolbar.setTool(Toolbar.HAND);
                toolbar.repaint();
                return;
            }
            ip = new ImagePlus("Montage"+num, slice);
            if (e.getClickCount() == 2){
                ip.show();
            }
            pressed = true;
        }
        public void mouseReleased(MouseEvent e) {
            pressed = false;
        }
        public void mouseExited(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}

        public void mouseDragged(MouseEvent event) {
            if (pressed){
                ip.copy(false);
            }
        }
        public void mouseMoved(MouseEvent e) {
        }
}


